@@ -12,7 +12,6 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
 import functools
 import hashlib
 import time
@@ -20,11 +19,12 @@ import warnings
 
 from oslo_log import log as logging
 import requests
+import simplejson
 import six
 from stevedore import driver as stevedore_driver
 
 from octavia.amphorae.driver_exceptions import exceptions as driver_except
-from octavia.amphorae.drivers import driver_base as driver_base
+from octavia.amphorae.drivers import driver_base
 from octavia.amphorae.drivers.haproxy import exceptions as exc
 from octavia.amphorae.drivers.keepalived import vrrp_rest_driver
 from octavia.common.config import cfg
@@ -32,7 +32,6 @@ from octavia.common import constants as consts
 from octavia.common.jinja.haproxy import jinja_cfg
 from octavia.common.tls_utils import cert_parser
 from octavia.common import utils
-from octavia.i18n import _LE, _LW
 
 LOG = logging.getLogger(__name__)
 API_VERSION = consts.API_VERSION
@@ -121,6 +120,8 @@ class HaproxyAmphoraLoadBalancerDriver(
             # compatibility with old amphora agent versions.
 
             port = amphorae_network_config.get(amphora.id).vrrp_port
+            LOG.debug("Post-VIP-Plugging with vrrp_ip %s vrrp_port %s",
+                      amphora.vrrp_ip, port.id)
             host_routes = [{'nexthop': hr.nexthop,
                             'destination': hr.destination}
                            for hr in subnet.host_routes]
@@ -135,9 +136,9 @@ class HaproxyAmphoraLoadBalancerDriver(
                                      load_balancer.vip.ip_address,
                                      net_info)
             except exc.Conflict:
-                LOG.warning(_LW('VIP with MAC {mac} already exists on '
-                                'amphora, skipping post_vip_plug').format(
-                    mac=port.mac_address))
+                LOG.warning('VIP with MAC %(mac)s already exists on amphora, '
+                            'skipping post_vip_plug',
+                            {'mac': port.mac_address})
 
     def post_network_plug(self, amphora, port):
         fixed_ips = []
@@ -155,12 +156,9 @@ class HaproxyAmphoraLoadBalancerDriver(
         try:
             self.client.plug_network(amphora, port_info)
         except exc.Conflict:
-            LOG.warning(_LW('Network with MAC {mac} already exists on '
-                            'amphora, skipping post_network_plug').format(
-                mac=port.mac_address))
-
-    def get_vrrp_interface(self, amphora):
-        return self.client.get_interface(amphora, amphora.vrrp_ip)['interface']
+            LOG.warning('Network with MAC %(mac)s already exists on amphora, '
+                        'skipping post_network_plug',
+                        {'mac': port.mac_address})
 
     def _process_tls_certificates(self, listener):
         """Processes TLS data from the listener.
@@ -184,7 +182,7 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         for cert in certs:
             pem = cert_parser.build_pem(cert)
-            md5 = hashlib.md5(six.b(pem)).hexdigest()  # nosec
+            md5 = hashlib.md5(pem).hexdigest()  # nosec
             name = '{cn}.pem'.format(cn=cert.primary_cn)
             self._apply(self._upload_cert, listener, pem, md5, name)
 
@@ -255,7 +253,7 @@ class AmphoraAPIClient(object):
         LOG.debug("request url %s", path)
         _request = getattr(self.session, method.lower())
         _url = self._base_url(amp.lb_network_ip) + path
-        LOG.debug("request url " + _url)
+        LOG.debug("request url %s", _url)
         timeout_tuple = (CONF.haproxy_amphora.rest_request_conn_timeout,
                          CONF.haproxy_amphora.rest_request_read_timeout)
         reqargs = {
@@ -267,7 +265,6 @@ class AmphoraAPIClient(object):
 
         headers['User-Agent'] = OCTAVIA_API_CLIENT
         self.ssl_adapter.uuid = amp.id
-        retry_attempt = False
         exception = None
         # Keep retrying
         for a in six.moves.xrange(CONF.haproxy_amphora.connection_max_retries):
@@ -278,22 +275,37 @@ class AmphoraAPIClient(object):
                         message="A true SSLContext object is not available"
                     )
                     r = _request(**reqargs)
-                LOG.debug("Connected to amphora. Response: {resp}".format(
-                    resp=r))
-                # Give a 404 response one retry.  Flask/werkzeug is
-                # returning 404 on startup.
-                if r.status_code == 404 and retry_attempt is False:
-                    retry_attempt = True
-                    raise requests.ConnectionError
+                LOG.debug('Connected to amphora. Response: %(resp)s',
+                          {'resp': r})
+
+                content_type = r.headers.get('content-type', '')
+                # Check the 404 to see if it is just that the network in the
+                # amphora is not yet up, in which case retry.
+                # Otherwise return the response quickly.
+                if r.status_code == 404:
+                    LOG.debug('Got a 404 (content-type: %(content_type)s) -- '
+                              'connection data: %(content)s',
+                              {'content_type': content_type,
+                               'content': r.content})
+                    if content_type.find("application/json") == -1:
+                        LOG.debug("Amphora agent not ready.")
+                        raise requests.ConnectionError
+                    try:
+                        json_data = r.json().get('details', '')
+                        if 'No suitable network interface found' in json_data:
+                            LOG.debug("Amphora network interface not found.")
+                            raise requests.ConnectionError
+                    except simplejson.JSONDecodeError:  # if r.json() fails
+                        pass  # TODO(rm_work) Should we do something?
                 return r
             except (requests.ConnectionError, requests.Timeout) as e:
                 exception = e
-                LOG.warning(_LW("Could not connect to instance. Retrying."))
+                LOG.warning("Could not connect to instance. Retrying.")
                 time.sleep(CONF.haproxy_amphora.connection_retry_interval)
 
-        LOG.error(_LE("Connection retries (currently set to %(max_retries)s) "
-                      "exhausted.  The amphora is unavailable. Reason: "
-                      "%(exception)s"),
+        LOG.error("Connection retries (currently set to %(max_retries)s) "
+                  "exhausted.  The amphora is unavailable. Reason: "
+                  "%(exception)s",
                   {'max_retries': CONF.haproxy_amphora.connection_max_retries,
                    'exception': exception})
         raise driver_except.TimeOutException()
@@ -340,7 +352,8 @@ class AmphoraAPIClient(object):
     def delete_listener(self, amp, listener_id):
         r = self.delete(
             amp, 'listeners/{listener_id}'.format(listener_id=listener_id))
-        return exc.check_exception(r)
+
+        return exc.check_exception(r, (404,))
 
     def get_info(self, amp):
         r = self.get(amp, "info")
@@ -362,7 +375,7 @@ class AmphoraAPIClient(object):
             amp,
             'listeners/{listener_id}/certificates/{filename}'.format(
                 listener_id=listener_id, filename=pem_filename))
-        return exc.check_exception(r)
+        return exc.check_exception(r, (404,))
 
     def plug_network(self, amp, port):
         r = self.post(amp, 'plug/network',
