@@ -99,6 +99,11 @@ class BaseController(rest.RestController):
         return self._get_db_obj(session, self.repositories.l7rule,
                                 data_models.L7Rule, id)
 
+    def _get_db_amp(self, session, id):
+        """Gets an Amphora from the database."""
+        return self._get_db_obj(session, self.repositories.amphora,
+                                data_models.Amphora, id)
+
     def _get_lb_project_id(self, session, id):
         """Get the project_id of the load balancer from the database."""
         lb = self._get_db_obj(session, self.repositories.load_balancer,
