@@ -114,9 +114,8 @@ class PoolsController(base.BaseController):
             return self.repositories.create_pool_on_load_balancer(
                 lock_session, pool_dict,
                 listener_id=listener_id)
-        except odb_exceptions.DBDuplicateEntry as de:
-            if ['id'] == de.columns:
-                raise exceptions.IDAlreadyExists()
+        except odb_exceptions.DBDuplicateEntry:
+            raise exceptions.IDAlreadyExists()
         except odb_exceptions.DBError:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
@@ -173,25 +172,24 @@ class PoolsController(base.BaseController):
                                    constants.RBAC_POST)
 
         lock_session = db_api.get_session(autocommit=False)
-        if self.repositories.check_quota_met(
-                context.session,
-                lock_session,
-                data_models.Pool,
-                pool.project_id):
-            lock_session.rollback()
-            raise exceptions.QuotaException
-
-        listener_repo = self.repositories.listener
-        pool_dict = db_prepare.create_pool(
-            pool.to_dict(render_unsets=True))
-
-        listener_id = pool_dict.pop('listener_id', None)
-        if listener_id:
-            if listener_repo.has_default_pool(lock_session,
-                                              listener_id):
-                raise exceptions.DuplicatePoolEntry()
-
         try:
+            if self.repositories.check_quota_met(
+                    context.session,
+                    lock_session,
+                    data_models.Pool,
+                    pool.project_id):
+                raise exceptions.QuotaException
+
+            listener_repo = self.repositories.listener
+            pool_dict = db_prepare.create_pool(
+                pool.to_dict(render_unsets=True))
+
+            listener_id = pool_dict.pop('listener_id', None)
+            if listener_id:
+                if listener_repo.has_default_pool(lock_session,
+                                                  listener_id):
+                    raise exceptions.DuplicatePoolEntry()
+
             self._test_lb_and_listener_statuses(
                 lock_session, lb_id=pool_dict['load_balancer_id'],
                 listener_ids=[listener_id] if listener_id else [])
@@ -290,6 +288,9 @@ class PoolsController(base.BaseController):
         self._auth_validate_action(context, db_pool.project_id,
                                    constants.RBAC_DELETE)
 
+        if db_pool.provisioning_status == constants.DELETED:
+            return
+
         self._test_lb_and_listener_statuses(
             context.session, lb_id=db_pool.load_balancer_id,
             listener_ids=self._get_affected_listener_ids(db_pool))
@@ -319,19 +320,14 @@ class PoolsController(base.BaseController):
         which controller, if any, should control be passed.
         """
         context = pecan.request.context.get('octavia_context')
-        if pool_id and len(remainder) and (remainder[0] == 'members' or
-                                           remainder[0] == 'healthmonitor'):
-            controller = remainder[0]
+        if pool_id and len(remainder) and remainder[0] == 'members':
             remainder = remainder[1:]
             db_pool = self.repositories.pool.get(context.session, id=pool_id)
             if not db_pool:
                 LOG.info("Pool %s not found.", pool_id)
                 raise exceptions.NotFound(resource=data_models.Pool._name(),
                                           id=pool_id)
-            if controller == 'members':
-                return member.MembersController(
-                    pool_id=db_pool.id), remainder
-            elif controller == 'healthmonitor':
-                return health_monitor.HealthMonitorController(
-                    load_balancer_id=db_pool.load_balancer_id,
-                    pool_id=db_pool.id), remainder
+            if remainder:
+                return member.MemberController(pool_id=db_pool.id), remainder
+            else:
+                return member.MembersController(pool_id=db_pool.id), remainder
