@@ -98,8 +98,9 @@ SUPPORTED_PROVISIONING_STATUSES = (ACTIVE, AMPHORA_ALLOCATED,
 MUTABLE_STATUSES = (ACTIVE,)
 DELETABLE_STATUSES = (ACTIVE, ERROR)
 
-SUPPORTED_AMPHORA_STATUSES = (AMPHORA_ALLOCATED, AMPHORA_BOOTING,
-                              AMPHORA_READY, DELETED, PENDING_DELETE)
+SUPPORTED_AMPHORA_STATUSES = (AMPHORA_ALLOCATED, AMPHORA_BOOTING, ERROR,
+                              AMPHORA_READY, DELETED,
+                              PENDING_CREATE, PENDING_DELETE)
 
 ONLINE = 'ONLINE'
 OFFLINE = 'OFFLINE'
@@ -278,6 +279,19 @@ CREATE_VRRP_SECURITY_RULES = 'octavia-create-vrrp-security-rules'
 
 GENERATE_SERVER_PEM_TASK = 'GenerateServerPEMTask'
 
+# Batch Member Update constants
+MEMBERS = 'members'
+UNORDERED_MEMBER_UPDATES_FLOW = 'octavia-unordered-member-updates-flow'
+UNORDERED_MEMBER_ACTIVE_FLOW = 'octavia-unordered-member-active-flow'
+UPDATE_ATTRIBUTES_FLOW = 'octavia-update-attributes-flow'
+DELETE_MODEL_OBJECT_FLOW = 'octavia-delete-model-object-flow'
+BATCH_UPDATE_MEMBERS_FLOW = 'octavia-batch-update-members-flow'
+MEMBER_TO_ERROR_ON_REVERT_FLOW = 'octavia-member-to-error-on-revert-flow'
+DECREMENT_MEMBER_QUOTA_FLOW = 'octavia-decrement-member-quota-flow'
+MARK_MEMBER_ACTIVE_INDB = 'octavia-mark-member-active-indb'
+UPDATE_MEMBER_INDB = 'octavia-update-member-indb'
+DELETE_MEMBER_INDB = 'octavia-delete-member-indb'
+
 # Task Names
 RELOAD_LB_AFTER_AMP_ASSOC = 'reload-lb-after-amp-assoc'
 RELOAD_LB_AFTER_AMP_ASSOC_FULL_GRAPH = 'reload-lb-after-amp-assoc-full-graph'
@@ -297,6 +311,7 @@ RPC_NAMESPACE_CONTROLLER_AGENT = 'controller'
 LB_CREATE_FAILOVER_PRIORITY = 20
 LB_CREATE_NORMAL_PRIORITY = 40
 LB_CREATE_SPARES_POOL_PRIORITY = 60
+LB_CREATE_ADMIN_FAILOVER_PRIORITY = 80
 BUILD_TYPE_PRIORITY = 'build_type_priority'
 
 # Active standalone roles and topology
@@ -434,6 +449,7 @@ DEFAULT_PAGE_SIZE = 1000
 
 # RBAC
 LOADBALANCER_API = 'os_load-balancer_api'
+RULE_API_ADMIN = 'rule:load-balancer:admin'
 RULE_API_READ = 'rule:load-balancer:read'
 RULE_API_READ_GLOBAL = 'rule:load-balancer:read-global'
 RULE_API_WRITE = 'rule:load-balancer:write'
@@ -448,8 +464,10 @@ RBAC_HEALTHMONITOR = '{}:healthmonitor:'.format(LOADBALANCER_API)
 RBAC_L7POLICY = '{}:l7policy:'.format(LOADBALANCER_API)
 RBAC_L7RULE = '{}:l7rule:'.format(LOADBALANCER_API)
 RBAC_QUOTA = '{}:quota:'.format(LOADBALANCER_API)
+RBAC_AMPHORA = '{}:amphora:'.format(LOADBALANCER_API)
 RBAC_POST = 'post'
 RBAC_PUT = 'put'
+RBAC_PUT_FAILOVER = 'put_failover'
 RBAC_DELETE = 'delete'
 RBAC_GET_ONE = 'get_one'
 RBAC_GET_ALL = 'get_all'
