@@ -22,6 +22,8 @@ import six
 from oslo_utils import excutils
 from webob import exc
 
+from octavia.i18n import _
+
 
 class OctaviaException(Exception):
     """Base Octavia Exception.
@@ -84,6 +86,11 @@ class InvalidOption(APIException):
     code = 400
 
 
+class InvalidFilterArgument(APIException):
+    msg = "One or more arguments are either duplicate or invalid"
+    code = 400
+
+
 class DisabledOption(APIException):
     msg = _("The selected %(option)s is not allowed in this deployment: "
             "%(value)s")
@@ -324,3 +331,7 @@ class InvalidMarker(APIException):
 class InvalidLimit(APIException):
     msg = _("Supplied pagination limit '%(key)s' is not valid.")
     code = 400
+
+
+class MissingVIPSecurityGroup(OctaviaException):
+    message = _('VIP security group is missing for load balancer: %(lb_id)s')
