@@ -905,3 +905,13 @@ class TestL7Rule(base.BaseAPITest):
                     params={'cascade': "true"})
         self.delete(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
                     status=409)
+
+    def test_delete_already_deleted(self):
+        l7rule = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api').get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(self.lb_id, status=constants.DELETED)
+        self.delete(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
+                    status=204)
