@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 import random
 
+import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
@@ -1050,6 +1050,26 @@ class TestListener(base.BaseAPITest):
             listener_id=api_listener['id'])
         self.delete(listener_path, status=409)
 
+    def test_delete_already_deleted(self):
+        lb = self.create_load_balancer(uuidutils.generate_uuid(),
+                                       name='lb1', description='desc1',
+                                       admin_state_up=False)
+        lb_id = lb['loadbalancer'].get('id')
+        self.set_lb_status(lb_id)
+        lb_listener = {'name': 'listener1', 'description': 'desc1',
+                       'admin_state_up': False,
+                       'protocol': constants.PROTOCOL_HTTP,
+                       'protocol_port': 80, 'connection_limit': 10,
+                       'loadbalancer_id': lb_id}
+        body = self._build_body(lb_listener)
+        api_listener = self.post(
+            self.LISTENERS_PATH, body).json.get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(lb_id, status=constants.DELETED)
+        listener_path = self.LISTENER_PATH.format(
+            listener_id=api_listener['id'])
+        self.delete(listener_path, status=204)
+
     def test_create_with_tls_termination_data(self):
         cert_id = uuidutils.generate_uuid()
         listener = self.create_listener(constants.PROTOCOL_TERMINATED_HTTPS,
@@ -1132,7 +1152,7 @@ class TestListener(base.BaseAPITest):
 
     def _getStats(self, listener_id):
         res = self.get(self.LISTENER_PATH.format(
-            listener_id=listener_id + "/stats"))
+                           listener_id=listener_id + "/stats"))
         return res.json.get('stats')
 
     def test_statistics(self):
@@ -1230,6 +1250,6 @@ class TestListener(base.BaseAPITest):
         with mock.patch.object(octavia.common.context.Context, 'project_id',
                                uuidutils.generate_uuid()):
             res = self.get(self.LISTENER_PATH.format(
-                listener_id=li['id'] + "/stats"), status=403)
+                               listener_id=li['id'] + "/stats"), status=403)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, res.json)
